%% Enter the current date
clear
%date = '11_04_tweeze1_no';

% Initialize an imageJResults array: copy into here from imageJ
%Be sure to put your standards first, in the right order
%After that, without a break, goes the data to calculate from
%This comes from the "mean" column, so it goes RGBRGBRGB etc.
imageJResults = [];
%Sample results: (divisible by 3):
%imageJResults = [7.5; 8.4; 9.3; 7.6; 8.5; 9.4; 7.7; 8.6; 9.5];

%% General section: sort output into R, G, and B columns
RGBresults = imageJResults(:,3);
redValues = zeros((length(RGBresults)/3), 1);
greenValues = zeros((length(RGBresults)/3), 1);
blueValues = zeros((length(RGBresults)/3), 1);
counter = 0; %Initialize the counter
for element = 1:length(imageJResults)
    if mod(counter, 3) == 0
        redValues(floor(counter/3)+1) = RGBresults(element);
    elseif mod(counter, 3) == 1
        greenValues(floor(counter/3)+1) = RGBresults(element);
    elseif mod(counter, 3) == 2
        blueValues(floor(counter/3)+1) = RGBresults(element);
    end
    counter = counter + 1; %increment the counter
end

RGB = zeros (length(redValues), 3);
RGB(:, 1) = redValues;
RGB(:, 2) = greenValues;
RGB(:, 3) = blueValues;

% Going forward, choose a section based on type of pH paper

%% pH 3 to 5.5 section
% Creates a calibration of R-G values vs standard pH
% The first 6 values of "redValues" should correspond to the
% standards below, in order. 'aah' = 'and a half'
 pH3to5aahStds = [3.0; 3.5; 4.0; 4.5; 5.0; 5.5];
 RminusG = redValues - greenValues;

y = RminusG(1:length(pH3to5aahStds));
quadFit = polyfit(pH3to5aahStds, y, 2);
a = quadFit(1);
b = quadFit(2);
c = quadFit(3);

%Now use the quadratic formula to calculate possible pH's
pHValues = zeros(length(RminusG)-length(pH3to5aahStds), 2);
for color = (length(pH3to5aahStds)+1:length(RminusG))
    d = c-RminusG(color);
    pHValues(color-length(pH3to5aahStds), 1) = ... 
        (-1*b + sqrt(b^2 - 4*a*d))/(2*a);
    pHValues(color-length(pH3to5aahStds), 2) = ... 
        (-1*b - sqrt(b^2 - 4*a*d))/(2*a);
%(-1*b - sqrt(b^2 - 4*a*(c-RminusG(color-length(pH3to5aahStds)))))/(2*a);
end
%save(date, 'redValues', 'greenValues', 'blueValues', 'pHValues')

% Next, find and run ONLY the section that corresponds to the pH range and
% color analysis method you wish to use

%% pH 6.0-8.0 quadratic section G-B

% Creates a calibration of (green-blue) vs standard pH
% The first 6 values of "green Values" should match below, in order.
pH6to8Stds = [6; 6.4; 6.8; 7; 7.2; 7.6; 8];
gMinusB = greenValues - blueValues;
X = [ones(length(pH6to8Stds),1) pH6to8Stds];
y = (gMinusB(1:length(pH6to8Stds)));
linearFit = X\y; %first element is intercept, second is slope

% Now that we have our calibration line, calculate pH from other data
pHValues = zeros(length(greenValues)-length(pH6to8Stds), 1);
for gValue = (length(pH6to8Stds)+1:length(greenValues))
    pHValues(gValue-length(pH6to8Stds)) = ...
        (gMinusB(gValue)-linearFit(1))/linearFit(2);
end
%% pH 1.0 to 2.5 section Red minus Green
% Create a quadratic calibration of R-G values vs pH
% This is the preferred method for this pH range
pH1to2aahStds = [1.0; 1.5; 2.0; 2.5];
rMinusG = redValues - greenValues;

y = rMinusG(1:length(pH1to2aahStds));
quadFit = polyfit(pH1to2aahStds, y, 2);
a = quadFit(1);
b = quadFit(2);
c = quadFit(3);

%Now use the quadratic formula to calculate possible pH's
pHValues = zeros(length(rMinusG)-length(pH1to2aahStds), 2);
for color = (length(pH1to2aahStds)+1:length(rMinusG))
    d = c-rMinusG(color);
    pHValues(color-length(pH1to2aahStds), 1) = ... 
        (-1*b + sqrt(b^2 - 4*a*d))/(2*a);
    pHValues(color-length(pH1to2aahStds), 2) = ... 
        (-1*b - sqrt(b^2 - 4*a*d))/(2*a);
%(-1*b - sqrt(b^2 - 4*a*(c-RminusG(color-length(pH3to5aahStds)))))/(2*a);
end
save(date, 'redValues', 'greenValues', 'blueValues', 'pHValues')


%% pH 1.0 to 2.5 section
% Create a quadratic calibration of R+B values vs pH
% This is typically an inferior method to the section above
% This method is not 1:1 and can yield imaginary resutls
pH1to2aahStds = [1.0; 1.5; 2.0; 2.5];
rPlusB = redValues + blueValues;

y = rPlusB(1:length(pH1to2aahStds));
quadFit = polyfit(pH1to2aahStds, y, 2);
a = quadFit(1);
b = quadFit(2);
c = quadFit(3);

%Now use the quadratic formula to calculate possible pH's
pHValues = zeros(length(rPlusB)-length(pH1to2aahStds), 2);
for color = (length(pH1to2aahStds)+1:length(rPlusB))
    d = c-rPlusB(color);
    pHValues(color-length(pH1to2aahStds), 1) = ... 
        (-1*b + sqrt(b^2 - 4*a*d))/(2*a);
    pHValues(color-length(pH1to2aahStds), 2) = ... 
        (-1*b - sqrt(b^2 - 4*a*d))/(2*a);
%(-1*b - sqrt(b^2 - 4*a*(c-RminusG(color-length(pH3to5aahStds)))))/(2*a);
end
save(date, 'redValues', 'greenValues', 'blueValues', 'pHValues')

%Other method:
% Creates a linear calibration of G-B values vs pH
%THIS DOES NOT WORK FOR HIGH NaCl SAMPLES SUCH AS SEAWATER
% pH1to2aahStds = [1.0; 1.5; 2.0; 2.5];
% gminusB = greenValues - blueValues;
% 
% X = [ones(length(pH1to2aahStds),1) pH1to2aahStds];
% y = gminusB(1:4);
% linearFit = X\y; %first element is intercept, second is slope
% 
% % Now that we have our calibration line, calculate pH from other data
% pHValues = zeros(length(greenValues)-length(pH1to2aahStds), 1);
% for gValue = (length(pH1to2aahStds)+1:length(greenValues))
%     pHValues(gValue-length(pH1to2aahStds)) = ...
%         (gminusB(gValue)-linearFit(1))/linearFit(2);
% end
% save(date, 'redValues', 'greenValues', 'blueValues', 'pHValues')

%% pH 2.5 to 4.5 section
pH2to4Stds = [2.5; 3.0; 3.3; 3.6; 3.9; 4.2; 4.5];
GminusB = greenValues - blueValues;

%X = [ones(length(pH2to4Stds),1) pH2to4Stds];
y = GminusB(1:length(pH2to4Stds));
quadFit = polyfit(pH2to4Stds, y, 2);
a = quadFit(1);
b = quadFit(2);
c = quadFit(3);

%Now use the quadratic formula to calculate possible pH's
pHValues = zeros(length(GminusB)-length(pH2to4Stds), 2);
for color = (length(pH2to4Stds)+1:length(GminusB))
    d = c-GminusB(color);
    pHValues(color-length(pH2to4Stds), 1) = ... 
        (-1*b + sqrt(b^2 - 4*a*d))/(2*a);
    pHValues(color-length(pH2to4Stds), 2) = ... 
        (-1*b - sqrt(b^2 - 4*a*d))/(2*a);
end
save(date, 'redValues', 'greenValues', 'blueValues', 'pHValues')

%% pH 6.0-8.0 section blue values
% Creates a calibration curve of blue vs pH
pH6to8Stds = [6; 6.4; 6.8; 7; 7.2; 7.6; 8];

y = blueValues(1:length(pH6to8Stds));
quadFit = polyfit(pH6to8Stds, y, 2);
a = quadFit(1);
b = quadFit(2);
c = quadFit(3);

%Now use the quadratic formula to calculate possible pH's
pHValues = zeros(length(blueValues)-length(pH6to8Stds), 2);
for color = (length(pH6to8Stds)+1:length(blueValues))
    d = c-blueValues(color);
    pHValues(color-length(pH6to8Stds), 1) = ... 
        (-1*b + sqrt(b^2 - 4*a*d))/(2*a);
    pHValues(color-length(pH6to8Stds), 2) = ... 
        (-1*b - sqrt(b^2 - 4*a*d))/(2*a);
end

%% MColorpHast pH 6.5-8.3 section
pH6p5to8p3Stds = [6.5; 6.8; 7.1; 7.4; 7.7; 7.9; 8.1; 8.3];

X = [ones(length(pH6p5to8p3Stds),1) pH6p5to8p3Stds];
y = (redValues(1:length(pH6p5to8p3Stds))-greenValues(1:length(pH6p5to8p3Stds)));
linearFit = X\y; %first element is intercept, second is slope

% Now that we have our calibration line, calculate pH from other data
pHValues = zeros(length(greenValues)-length(pH6p5to8p3Stds), 1);
for gValue = (length(pH6p5to8p3Stds)+1:length(greenValues))
    pHValues(gValue-length(pH6p5to8p3Stds)) = ...
        (redValues(gValue)-greenValues(gValue)-linearFit(1))/linearFit(2);
end
save(date, 'redValues', 'greenValues', 'blueValues', 'pHValues')
%% Universal pH paper section
% For this section, the calibration must be split into two parts
% These are 2 quadratic curves, from pH 1-6 and pH 7-10
% If your pH falls in between 6-7, try using pH 6.0-8.0 paper instead

%% Universal pH paper range 1-6
pH1to6Stds = [1; 2; 3; 4; 5; 6];
RminusG = redValues - greenValues;

y = RminusG(1:length(pH1to6Stds));
quadFit = polyfit(pH1to6Stds, y, 2);
a = quadFit(1);
b = quadFit(2);
c = quadFit(3);

%Now use the quadratic formula to calculate possible pH's
pHValues = zeros(length(RminusG)-length(pH1to6Stds), 2);
for color = (length(pH1to6Stds)+1:length(RminusG))
    d = c-RminusG(color);
    pHValues(color-length(pH1to6Stds), 1) = ... 
        (-1*b + sqrt(b^2 - 4*a*d))/(2*a);
    pHValues(color-length(pH1to6Stds), 2) = ... 
        (-1*b - sqrt(b^2 - 4*a*d))/(2*a);
end
save(date, 'redValues', 'greenValues', 'blueValues', 'pHValues')
%% Universal pH paper range 7-11
pH7to11Stds = [7;8;9;10;11];
GminusB = greenValues - blueValues;

y = GminusB(1:length(pH7to11Stds));
quadFit = polyfit(pH7to11Stds, y, 2);
a = quadFit(1);
b = quadFit(2);
c = quadFit(3);

%Now use the quadratic formula to calculate possible pH's
pHValues = zeros(length(GminusB)-length(pH7to11Stds), 2);
for color = (length(pH7to11Stds)+1:length(GminusB))
    d = c-GminusB(color);
    pHValues(color-length(pH7to11Stds), 1) = ... 
        (-1*b + sqrt(b^2 - 4*a*d))/(2*a);
    pHValues(color-length(pH7to11Stds), 2) = ... 
        (-1*b - sqrt(b^2 - 4*a*d))/(2*a);
end
save(date, 'redValues', 'greenValues', 'blueValues', 'pHValues')
